@@ -100,30 +100,32 @@ parse (BzYamlParser   *self,
        gboolean        toplevel,
        SchemaNodeData *schema,
        GHashTable     *output,
+       GHashTable     *anchors,
        GPtrArray      *path_stack,
        GError        **error);
 
 static char *
 join_path_stack (GPtrArray *path_stack);
 
-static GValue *
-parse_scalar (const GVariantType *vtype,
-              const guchar       *data,
-              yaml_event_t       *event,
-              GError            **error);
-
 static GObject *
 parse_object (BzYamlParser  *self,
               yaml_parser_t *parser,
               yaml_event_t  *event,
               GType          object_gtype,
               GHashTable    *type_hints,
+              GHashTable    *anchors,
               const char    *prop_path,
               GError       **error);
 
 static void
 destroy_gvalue (GValue *value);
 
+static GValue *
+make_gvalue_alloc (GType type);
+
+static GValue *
+copy_gvalue_alloc (GValue *value);
+
 static void
 bz_yaml_parser_dispose (GObject *object)
 {
@@ -161,6 +163,7 @@ bz_yaml_parser_real_process_bytes (BzParser *iface_self,
   yaml_parser_t parser             = { 0 };
   yaml_event_t  event              = { 0 };
   g_autoptr (GHashTable) output    = NULL;
+  g_autoptr (GHashTable) anchors   = NULL;
   g_autoptr (GPtrArray) path_stack = NULL;
   gboolean result                  = FALSE;
 
@@ -175,6 +178,9 @@ bz_yaml_parser_real_process_bytes (BzParser *iface_self,
   output = g_hash_table_new_full (
       g_str_hash, g_str_equal,
       g_free, (GDestroyNotify) destroy_gvalue);
+  anchors = g_hash_table_new_full (
+      g_str_hash, g_str_equal,
+      g_free, (GDestroyNotify) destroy_gvalue);
   path_stack = g_ptr_array_new_with_free_func (g_free);
 
   result = parse (
@@ -185,6 +191,7 @@ bz_yaml_parser_real_process_bytes (BzParser *iface_self,
       TRUE,
       self->schema,
       output,
+      anchors,
       path_stack,
       &local_error);
   yaml_parser_delete (&parser);
@@ -398,6 +405,7 @@ parse (BzYamlParser   *self,
        gboolean        toplevel,
        SchemaNodeData *schema,
        GHashTable     *output,
+       GHashTable     *anchors,
        GPtrArray      *path_stack,
        GError        **error)
 {
@@ -448,6 +456,46 @@ parse (BzYamlParser   *self,
       return FALSE;                                                          \
     }
 
+#define TRY_ALIAS(var, gtype, string_type)                                       \
+  if (event->type == YAML_ALIAS_EVENT)                                           \
+    {                                                                            \
+      GValue *_value = NULL;                                                     \
+                                                                                 \
+      _value = g_hash_table_lookup (                                             \
+          anchors,                                                               \
+          event->data.alias.anchor);                                             \
+      if (_value == NULL)                                                        \
+        {                                                                        \
+          g_set_error (                                                          \
+              error,                                                             \
+              BZ_YAML_ERROR,                                                     \
+              BZ_YAML_ERROR_INVALID_YAML,                                        \
+              "Failed to resolve YAML alias at line %zu, column %zu: "           \
+              "the anchor \"%s\" does not yet exist",                            \
+              event->start_mark.line,                                            \
+              event->start_mark.column,                                          \
+              (const char *) event->data.alias.anchor);                          \
+          yaml_event_delete (event);                                             \
+          return FALSE;                                                          \
+        }                                                                        \
+                                                                                 \
+      if (!G_VALUE_HOLDS (_value, (gtype)))                                      \
+        {                                                                        \
+          g_set_error (                                                          \
+              error,                                                             \
+              BZ_YAML_ERROR,                                                     \
+              BZ_YAML_ERROR_DOES_NOT_CONFORM,                                    \
+              "Failed to validate YAML against schema at line %zu, column %zu: " \
+              "the alias needs to be of type " string_type " here",              \
+              event->start_mark.line,                                            \
+              event->start_mark.column);                                         \
+          yaml_event_delete (event);                                             \
+          return FALSE;                                                          \
+        }                                                                        \
+                                                                                 \
+      (var) = copy_gvalue_alloc (_value);                                        \
+    }
+
   if (toplevel)
     {
       EXPECT (YAML_STREAM_START_EVENT, "start of stream");
@@ -462,17 +510,74 @@ parse (BzYamlParser   *self,
       {
         GValue *value = NULL;
 
-        EXPECT (YAML_SCALAR_EVENT, "scalar");
-
-        value = parse_scalar (
-            (const GVariantType *) schema->scalar.vtype,
-            event->data.scalar.value,
-            event,
-            error);
-        if (value == NULL)
+        TRY_ALIAS (value, G_TYPE_VARIANT, "scalar");
+        if (value != NULL)
           {
-            yaml_event_delete (event);
-            return FALSE;
+            GVariant *variant = NULL;
+
+            variant = g_value_get_variant (value);
+            if (!g_variant_type_equal (
+                    g_variant_get_type (variant),
+                    (const GVariantType *) schema->scalar.vtype))
+              {
+                g_set_error (
+                    error,
+                    BZ_YAML_ERROR,
+                    BZ_YAML_ERROR_DOES_NOT_CONFORM,
+                    "Failed to validate YAML against schema at line %zu, column %zu: "
+                    "the alias needs to be of scalar type %s here",
+                    event->start_mark.line,
+                    event->start_mark.column,
+                    schema->scalar.vtype);
+                destroy_gvalue (value);
+                yaml_event_delete (event);
+                return FALSE;
+              }
+          }
+        else
+          {
+            g_autofree char *anchor      = NULL;
+            g_autoptr (GVariant) variant = NULL;
+
+            EXPECT (YAML_SCALAR_EVENT, "scalar");
+            anchor = bz_maybe_strdup ((const char *) event->data.scalar.anchor);
+
+            if (g_variant_type_equal ((const GVariantType *) schema->scalar.vtype, G_VARIANT_TYPE_STRING))
+              variant = g_variant_new_string ((const char *) event->data.scalar.value);
+            else
+              {
+                g_autoptr (GError) local_error = NULL;
+
+                variant = g_variant_parse (
+                    (const GVariantType *) schema->scalar.vtype,
+                    (const char *) event->data.scalar.value,
+                    NULL,
+                    NULL,
+                    &local_error);
+                if (variant == NULL)
+                  {
+                    g_set_error (
+                        error,
+                        BZ_YAML_ERROR,
+                        BZ_YAML_ERROR_BAD_SCALAR,
+                        "Failed to parse scalar variant at line %zu, column %zu: "
+                        "%s",
+                        event->start_mark.line,
+                        event->start_mark.column,
+                        local_error->message);
+                    yaml_event_delete (event);
+                    return FALSE;
+                  }
+              }
+
+            value = make_gvalue_alloc (G_TYPE_VARIANT);
+            g_value_set_variant (value, g_steal_pointer (&variant));
+
+            if (anchor != NULL)
+              g_hash_table_replace (
+                  anchors,
+                  g_steal_pointer (&anchor),
+                  copy_gvalue_alloc (value));
           }
 
         g_hash_table_replace (output, join_path_stack (path_stack), value);
@@ -483,72 +588,89 @@ parse (BzYamlParser   *self,
         g_autoptr (GObject) object = NULL;
         GValue *value              = NULL;
 
-        object = parse_object (
-            self,
-            parser,
-            event,
-            schema->object.type,
-            schema->object.type_hints,
-            NULL,
-            error);
-        if (object == NULL)
-          /* event is already cleaned up */
-          return FALSE;
+        TRY_ALIAS (value, schema->object.type, "object");
+        if (value == NULL)
+          {
+            object = parse_object (
+                self,
+                parser,
+                event,
+                schema->object.type,
+                schema->object.type_hints,
+                anchors,
+                NULL,
+                error);
+            if (object == NULL)
+              /* event is already cleaned up */
+              return FALSE;
+
+            value = make_gvalue_alloc (schema->object.type);
+            g_value_set_object (value, object);
+          }
 
-        value = g_new0 (typeof (*value), 1);
-        g_value_init (value, schema->object.type);
-        g_value_set_object (value, object);
         g_hash_table_replace (output, join_path_stack (path_stack), value);
       }
       break;
     case KIND_LIST:
       {
-        g_autoptr (GPtrArray) list = NULL;
-        GValue *value              = NULL;
+        GValue *value = NULL;
 
-        EXPECT (YAML_SEQUENCE_START_EVENT, "list");
-
-        list = g_ptr_array_new_with_free_func ((GDestroyNotify) destroy_gvalue);
-
-        for (;;)
+        TRY_ALIAS (value, G_TYPE_PTR_ARRAY, "list");
+        if (value == NULL)
           {
-            g_autoptr (GHashTable) list_output    = NULL;
-            g_autoptr (GPtrArray) list_path_stack = NULL;
-            gboolean result                       = FALSE;
-            GValue  *append                       = NULL;
+            g_autofree char *anchor    = NULL;
+            g_autoptr (GPtrArray) list = NULL;
 
-            NEXT_EVENT ();
-            if (event->type == YAML_SEQUENCE_END_EVENT)
-              break;
+            EXPECT (YAML_SEQUENCE_START_EVENT, "list");
+            anchor = bz_maybe_strdup ((const char *) event->data.sequence_start.anchor);
 
-            list_output = g_hash_table_new_full (
-                g_str_hash, g_str_equal,
-                g_free, (GDestroyNotify) destroy_gvalue);
-            list_path_stack = g_ptr_array_new_with_free_func (g_free);
+            list = g_ptr_array_new_with_free_func ((GDestroyNotify) destroy_gvalue);
+            for (;;)
+              {
+                g_autoptr (GHashTable) list_output    = NULL;
+                g_autoptr (GPtrArray) list_path_stack = NULL;
+                gboolean result                       = FALSE;
+                GValue  *append                       = NULL;
 
-            result = parse (
-                self,
-                parser,
-                event,
-                FALSE,
-                FALSE,
-                schema->list.child,
-                list_output,
-                list_path_stack,
-                error);
-            if (!result)
-              /* event is already cleaned up */
-              return FALSE;
+                NEXT_EVENT ();
+                if (event->type == YAML_SEQUENCE_END_EVENT)
+                  break;
 
-            append = g_new0 (typeof (*append), 1);
-            g_value_init (append, G_TYPE_HASH_TABLE);
-            g_value_set_boxed (append, list_output);
-            g_ptr_array_add (list, append);
+                list_output = g_hash_table_new_full (
+                    g_str_hash, g_str_equal,
+                    g_free, (GDestroyNotify) destroy_gvalue);
+                list_path_stack = g_ptr_array_new_with_free_func (g_free);
+
+                result = parse (
+                    self,
+                    parser,
+                    event,
+                    FALSE,
+                    FALSE,
+                    schema->list.child,
+                    list_output,
+                    anchors,
+                    list_path_stack,
+                    error);
+                if (!result)
+                  /* event is already cleaned up */
+                  return FALSE;
+
+                append = make_gvalue_alloc (G_TYPE_HASH_TABLE);
+                g_value_set_boxed (append, list_output);
+                g_ptr_array_add (list, append);
+              }
+
+            value = make_gvalue_alloc (G_TYPE_PTR_ARRAY);
+            g_value_set_boxed (value, list);
+
+            if (anchor != NULL)
+              g_hash_table_replace (
+                  anchors,
+                  g_steal_pointer (&anchor),
+                  copy_gvalue_alloc (value));
           }
 
-        value = g_new0 (typeof (*value), 1);
-        g_value_init (value, G_TYPE_PTR_ARRAY);
-        g_value_set_boxed (value, list);
         g_hash_table_replace (output, join_path_stack (path_stack), value);
       }
       break;
@@ -599,6 +721,7 @@ parse (BzYamlParser   *self,
                 FALSE,
                 map_schema,
                 output,
+                anchors,
                 path_stack,
                 error);
             if (!result)
@@ -645,61 +768,31 @@ join_path_stack (GPtrArray *path_stack)
   return g_string_free_and_steal (string);
 }
 
-static GValue *
-parse_scalar (const GVariantType *vtype,
-              const guchar       *data,
-              yaml_event_t       *event,
-              GError            **error)
-{
-  g_autoptr (GError) local_error = NULL;
-  g_autoptr (GVariant) variant   = NULL;
-  GValue *value                  = NULL;
-
-  if (g_variant_type_equal (vtype, G_VARIANT_TYPE_STRING))
-    variant = g_variant_new_string ((const char *) data);
-  else
-    {
-      variant = g_variant_parse (
-          vtype,
-          (const char *) data,
-          NULL,
-          NULL,
-          &local_error);
-      if (variant == NULL)
-        {
-          g_set_error (
-              error,
-              BZ_YAML_ERROR,
-              BZ_YAML_ERROR_BAD_SCALAR,
-              "Failed to parse scalar variant at line %zu, column %zu: "
-              "%s",
-              event->start_mark.line,
-              event->start_mark.column,
-              local_error->message);
-          return NULL;
-        }
-    }
-
-  value = g_new0 (typeof (*value), 1);
-  g_value_init (value, G_TYPE_VARIANT);
-  g_value_set_variant (value, g_steal_pointer (&variant));
-
-  return value;
-}
-
 static GObject *
 parse_object (BzYamlParser  *self,
               yaml_parser_t *parser,
               yaml_event_t  *event,
               GType          object_gtype,
               GHashTable    *type_hints,
+              GHashTable    *anchors,
               const char    *prop_path,
               GError       **error)
 {
+  GValue          *value             = NULL;
+  g_autofree char *object_anchor     = NULL;
   g_autoptr (GTypeClass) gtype_class = NULL;
   g_autoptr (GObject) object         = NULL;
 
+  TRY_ALIAS (value, object_gtype, "object mapping");
+  if (value != NULL)
+    {
+      object = g_value_dup_object (value);
+      destroy_gvalue (value);
+      return g_steal_pointer (&object);
+    }
+
   EXPECT (YAML_MAPPING_START_EVENT, "object mapping");
+  object_anchor = bz_maybe_strdup ((const char *) event->data.mapping_start.anchor);
 
   gtype_class = g_type_class_ref (object_gtype);
   object      = g_object_new (object_gtype, NULL);
@@ -741,59 +834,98 @@ parse_object (BzYamlParser  *self,
           GType            element_gtype     = 0;
           g_autoptr (GListModel) list        = NULL;
 
-          EXPECT (YAML_SEQUENCE_START_EVENT, "sequence");
-
           if (prop_path != NULL)
             replace_prop_path = g_strdup_printf ("%s.%s", prop_path, property);
-
           element_gtype = GPOINTER_TO_SIZE (g_hash_table_lookup (
               type_hints, replace_prop_path != NULL ? replace_prop_path : property));
 
-          if (element_gtype == GTK_TYPE_STRING_OBJECT)
+          TRY_ALIAS (value, G_TYPE_LIST_MODEL, "mappings");
+          if (value != NULL)
             {
-              list = (GListModel *) gtk_string_list_new (NULL);
+              list = g_value_dup_object (value);
 
-              for (;;)
+              if (!G_IS_LIST_MODEL (list) ||
+                  g_list_model_get_item_type (list) != element_gtype)
                 {
-                  NEXT_EVENT ();
-
-                  if (event->type == YAML_SEQUENCE_END_EVENT)
-                    break;
-                  EXPECT (YAML_SCALAR_EVENT, "scalar list value");
-
-                  gtk_string_list_append (
-                      GTK_STRING_LIST (list),
-                      (const char *) event->data.scalar.value);
+                  g_set_error (
+                      error,
+                      BZ_YAML_ERROR,
+                      BZ_YAML_ERROR_DOES_NOT_CONFORM,
+                      "Failed to validate YAML against schema at line %zu, column %zu: "
+                      "the alias needs to be a list of object type %s here",
+                      event->start_mark.line,
+                      event->start_mark.column,
+                      g_type_name (element_gtype));
+                  destroy_gvalue (value);
+                  yaml_event_delete (event);
+                  return FALSE;
                 }
             }
           else
             {
-              if (element_gtype == 0)
-                element_gtype = G_TYPE_OBJECT;
+              g_autofree char *anchor = NULL;
 
-              list = (GListModel *) g_list_store_new (element_gtype);
+              EXPECT (YAML_SEQUENCE_START_EVENT, "sequence");
+              anchor = bz_maybe_strdup ((const char *) event->data.sequence_start.anchor);
 
-              for (;;)
+              if (element_gtype == GTK_TYPE_STRING_OBJECT)
                 {
-                  g_autoptr (GObject) child_object = NULL;
+                  list = (GListModel *) gtk_string_list_new (NULL);
 
-                  NEXT_EVENT ();
-                  if (event->type == YAML_SEQUENCE_END_EVENT)
-                    break;
+                  for (;;)
+                    {
+                      NEXT_EVENT ();
 
-                  child_object = parse_object (
-                      self,
-                      parser,
-                      event,
-                      element_gtype,
-                      type_hints,
-                      replace_prop_path != NULL ? replace_prop_path : property,
-                      error);
-                  if (child_object == NULL)
-                    /* event is already cleaned up */
-                    return FALSE;
+                      if (event->type == YAML_SEQUENCE_END_EVENT)
+                        break;
+                      EXPECT (YAML_SCALAR_EVENT, "scalar list value");
 
-                  g_list_store_append (G_LIST_STORE (list), child_object);
+                      gtk_string_list_append (
+                          GTK_STRING_LIST (list),
+                          (const char *) event->data.scalar.value);
+                    }
+                }
+              else
+                {
+                  if (element_gtype == 0)
+                    element_gtype = G_TYPE_OBJECT;
+
+                  list = (GListModel *) g_list_store_new (element_gtype);
+
+                  for (;;)
+                    {
+                      g_autoptr (GObject) child_object = NULL;
+
+                      NEXT_EVENT ();
+                      if (event->type == YAML_SEQUENCE_END_EVENT)
+                        break;
+
+                      child_object = parse_object (
+                          self,
+                          parser,
+                          event,
+                          element_gtype,
+                          type_hints,
+                          anchors,
+                          replace_prop_path != NULL ? replace_prop_path : property,
+                          error);
+                      if (child_object == NULL)
+                        /* event is already cleaned up */
+                        return FALSE;
+
+                      g_list_store_append (G_LIST_STORE (list), child_object);
+                    }
+                }
+
+              if (anchor != NULL)
+                {
+                  value = make_gvalue_alloc (G_TYPE_LIST_MODEL);
+                  g_value_set_object (value, list);
+
+                  g_hash_table_replace (
+                      anchors,
+                      g_steal_pointer (&anchor),
+                      value);
                 }
             }
 
@@ -813,6 +945,7 @@ parse_object (BzYamlParser  *self,
               event,
               spec->value_type,
               type_hints,
+              anchors,
               replace_prop_path != NULL ? replace_prop_path : property,
               error);
           if (prop_object == NULL)
@@ -826,33 +959,56 @@ parse_object (BzYamlParser  *self,
           g_autoptr (GEnumClass) class = NULL;
           GEnumValue *enum_value       = NULL;
 
-          EXPECT (YAML_SCALAR_EVENT, "scalar enum value");
-
-          class = g_type_class_ref (spec->value_type);
-
-          enum_value = g_enum_get_value_by_nick (
-              class, (const char *) event->data.scalar.value);
-          if (enum_value == NULL)
-            enum_value = g_enum_get_value_by_name (
-                class, (const char *) event->data.scalar.value);
-
-          if (enum_value == NULL)
+          TRY_ALIAS (value, spec->value_type, "scalar enum value");
+          if (value != NULL)
             {
-              g_set_error (
-                  error,
-                  BZ_YAML_ERROR,
-                  BZ_YAML_ERROR_BAD_SCALAR,
-                  "Failed to parse scalar enum at line %zu, column %zu: "
-                  "'%s' does not exist in type %s",
-                  event->start_mark.line,
-                  event->start_mark.column,
-                  (const char *) event->data.scalar.value,
-                  g_type_name (spec->value_type));
-              yaml_event_delete (event);
-              return FALSE;
+              g_object_set_property (object, property, value);
+              destroy_gvalue (value);
             }
+          else
+            {
+              g_autofree char *anchor = NULL;
 
-          g_object_set (object, property, enum_value->value, NULL);
+              EXPECT (YAML_SCALAR_EVENT, "scalar enum value");
+              anchor = bz_maybe_strdup ((const char *) event->data.scalar.anchor);
+
+              class = g_type_class_ref (spec->value_type);
+
+              enum_value = g_enum_get_value_by_nick (
+                  class, (const char *) event->data.scalar.value);
+              if (enum_value == NULL)
+                enum_value = g_enum_get_value_by_name (
+                    class, (const char *) event->data.scalar.value);
+
+              if (enum_value == NULL)
+                {
+                  g_set_error (
+                      error,
+                      BZ_YAML_ERROR,
+                      BZ_YAML_ERROR_BAD_SCALAR,
+                      "Failed to parse scalar enum at line %zu, column %zu: "
+                      "'%s' does not exist in type %s",
+                      event->start_mark.line,
+                      event->start_mark.column,
+                      (const char *) event->data.scalar.value,
+                      g_type_name (spec->value_type));
+                  yaml_event_delete (event);
+                  return FALSE;
+                }
+
+              if (anchor != NULL)
+                {
+                  value = make_gvalue_alloc (spec->value_type);
+                  g_value_set_enum (value, enum_value->value);
+
+                  g_hash_table_replace (
+                      anchors,
+                      g_steal_pointer (&anchor),
+                      value);
+                }
+
+              g_object_set (object, property, enum_value->value, NULL);
+            }
         }
       else
         {
@@ -860,89 +1016,135 @@ parse_object (BzYamlParser  *self,
           const GVariantType *vtype      = NULL;
           g_autoptr (GVariant) variant   = NULL;
 
-          EXPECT (YAML_SCALAR_EVENT, "scalar value");
-
-          switch (spec->value_type)
+          TRY_ALIAS (value, spec->value_type, "scalar");
+          if (value != NULL)
             {
-            case G_TYPE_BOOLEAN:
-              vtype = G_VARIANT_TYPE_BOOLEAN;
-              break;
-            case G_TYPE_INT:
-              vtype = G_VARIANT_TYPE_INT32;
-              break;
-            case G_TYPE_INT64:
-              vtype = G_VARIANT_TYPE_INT64;
-              break;
-            case G_TYPE_UINT:
-              vtype = G_VARIANT_TYPE_UINT32;
-              break;
-            case G_TYPE_UINT64:
-              vtype = G_VARIANT_TYPE_UINT64;
-              break;
-            case G_TYPE_DOUBLE:
-            case G_TYPE_FLOAT:
-              vtype = G_VARIANT_TYPE_DOUBLE;
-              break;
-            case G_TYPE_STRING:
-            default:
-              vtype = G_VARIANT_TYPE_STRING;
-              break;
+              g_object_set_property (object, property, value);
+              destroy_gvalue (value);
             }
-
-          if (g_variant_type_equal (vtype, G_VARIANT_TYPE_STRING))
-            variant = g_variant_new_string ((const char *) event->data.scalar.value);
           else
             {
-              variant = g_variant_parse (
-                  vtype,
-                  (const char *) event->data.scalar.value,
-                  NULL,
-                  NULL,
-                  &local_error);
-              if (variant == NULL)
-                {
-                  g_set_error (
-                      error,
-                      BZ_YAML_ERROR,
-                      BZ_YAML_ERROR_BAD_SCALAR,
-                      "Failed to parse scalar variant at line %zu, column %zu: %s",
-                      event->start_mark.line,
-                      event->start_mark.column,
-                      local_error->message);
-                  yaml_event_delete (event);
-                  return FALSE;
-                }
-            }
+              g_autofree char *anchor = NULL;
 
-          switch (spec->value_type)
-            {
-            case G_TYPE_BOOLEAN:
-              g_object_set (object, property, g_variant_get_boolean (variant), NULL);
-              break;
-            case G_TYPE_INT:
-              g_object_set (object, property, g_variant_get_int32 (variant), NULL);
-              break;
-            case G_TYPE_INT64:
-              g_object_set (object, property, g_variant_get_int64 (variant), NULL);
-              break;
-            case G_TYPE_UINT:
-              g_object_set (object, property, g_variant_get_uint32 (variant), NULL);
-              break;
-            case G_TYPE_UINT64:
-              g_object_set (object, property, g_variant_get_uint64 (variant), NULL);
-              break;
-            case G_TYPE_DOUBLE:
-            case G_TYPE_FLOAT:
-              g_object_set (object, property, g_variant_get_double (variant), NULL);
-              break;
-            case G_TYPE_STRING:
-            default:
-              g_object_set (object, property, g_variant_get_string (variant, NULL), NULL);
-              break;
+              EXPECT (YAML_SCALAR_EVENT, "scalar value");
+              anchor = bz_maybe_strdup ((const char *) event->data.scalar.anchor);
+
+              switch (spec->value_type)
+                {
+                case G_TYPE_BOOLEAN:
+                  vtype = G_VARIANT_TYPE_BOOLEAN;
+                  break;
+                case G_TYPE_INT:
+                  vtype = G_VARIANT_TYPE_INT32;
+                  break;
+                case G_TYPE_INT64:
+                  vtype = G_VARIANT_TYPE_INT64;
+                  break;
+                case G_TYPE_UINT:
+                  vtype = G_VARIANT_TYPE_UINT32;
+                  break;
+                case G_TYPE_UINT64:
+                  vtype = G_VARIANT_TYPE_UINT64;
+                  break;
+                case G_TYPE_DOUBLE:
+                case G_TYPE_FLOAT:
+                  vtype = G_VARIANT_TYPE_DOUBLE;
+                  break;
+                case G_TYPE_STRING:
+                default:
+                  vtype = G_VARIANT_TYPE_STRING;
+                  break;
+                }
+
+              if (g_variant_type_equal (vtype, G_VARIANT_TYPE_STRING))
+                variant = g_variant_new_string ((const char *) event->data.scalar.value);
+              else
+                {
+                  variant = g_variant_parse (
+                      vtype,
+                      (const char *) event->data.scalar.value,
+                      NULL,
+                      NULL,
+                      &local_error);
+                  if (variant == NULL)
+                    {
+                      g_set_error (
+                          error,
+                          BZ_YAML_ERROR,
+                          BZ_YAML_ERROR_BAD_SCALAR,
+                          "Failed to parse scalar variant at line %zu, column %zu: %s",
+                          event->start_mark.line,
+                          event->start_mark.column,
+                          local_error->message);
+                      yaml_event_delete (event);
+                      return FALSE;
+                    }
+                }
+
+              if (anchor != NULL)
+                value = make_gvalue_alloc (spec->value_type);
+
+              switch (spec->value_type)
+                {
+                case G_TYPE_BOOLEAN:
+                  g_object_set (object, property, g_variant_get_boolean (variant), NULL);
+                  if (anchor != NULL)
+                    g_value_set_boolean (value, g_variant_get_boolean (variant));
+                  break;
+                case G_TYPE_INT:
+                  g_object_set (object, property, g_variant_get_int32 (variant), NULL);
+                  if (anchor != NULL)
+                    g_value_set_int (value, g_variant_get_int32 (variant));
+                  break;
+                case G_TYPE_INT64:
+                  g_object_set (object, property, g_variant_get_int64 (variant), NULL);
+                  if (anchor != NULL)
+                    g_value_set_int64 (value, g_variant_get_int64 (variant));
+                  break;
+                case G_TYPE_UINT:
+                  g_object_set (object, property, g_variant_get_uint32 (variant), NULL);
+                  if (anchor != NULL)
+                    g_value_set_uint (value, g_variant_get_uint32 (variant));
+                  break;
+                case G_TYPE_UINT64:
+                  g_object_set (object, property, g_variant_get_uint64 (variant), NULL);
+                  if (anchor != NULL)
+                    g_value_set_uint64 (value, g_variant_get_uint64 (variant));
+                  break;
+                case G_TYPE_DOUBLE:
+                case G_TYPE_FLOAT:
+                  g_object_set (object, property, g_variant_get_double (variant), NULL);
+                  if (anchor != NULL)
+                    g_value_set_double (value, g_variant_get_double (variant));
+                  break;
+                case G_TYPE_STRING:
+                default:
+                  g_object_set (object, property, g_variant_get_string (variant, NULL), NULL);
+                  if (anchor != NULL)
+                    g_value_set_string (value, g_variant_get_string (variant, NULL));
+                  break;
+                }
+
+              if (anchor != NULL)
+                g_hash_table_replace (
+                    anchors,
+                    g_steal_pointer (&anchor),
+                    value);
             }
         }
     }
 
+  if (object_anchor != NULL)
+    {
+      value = make_gvalue_alloc (G_TYPE_LIST_MODEL);
+      g_value_set_object (value, object);
+
+      g_hash_table_replace (
+          anchors,
+          g_steal_pointer (&object_anchor),
+          value);
+    }
+
   return g_steal_pointer (&object);
 }
 
@@ -976,3 +1178,24 @@ destroy_gvalue (GValue *value)
   g_value_unset (value);
   g_free (value);
 }
+
+static GValue *
+make_gvalue_alloc (GType type)
+{
+  GValue *ret = NULL;
+
+  ret = g_new0 (typeof (*ret), 1);
+  g_value_init (ret, type);
+  return ret;
+}
+
+static GValue *
+copy_gvalue_alloc (GValue *value)
+{
+  GValue *ret = NULL;
+
+  ret = g_new0 (typeof (*ret), 1);
+  g_value_init (ret, value->g_type);
+  g_value_copy (value, ret);
+  return ret;
+}
